"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.DaemonClient = void 0;
const imperative_1 = require("@zowe/imperative");
/**
 * Class for handling client connections to our persistent service (e.g. daemon mode)
 * @export
 * @class DaemonClient
 */
class DaemonClient {
    /**
     * Creates an instance of DaemonClient.
     * @param {net.Socket} mClient
     * @param {net.Server} mServer
     * @memberof DaemonClient
     */
    constructor(mClient, mServer) {
        this.mClient = mClient;
        this.mServer = mServer;
    }
    /**
     * Run an instance of this client and wait for proper events
     * @memberof DaemonClient
     */
    run() {
        imperative_1.Imperative.api.appLogger.trace('daemon client connected');
        this.mClient.on('end', this.end.bind(this));
        this.mClient.on('close', this.close.bind(this));
        this.mClient.on('data', this.data.bind(this));
    }
    /**
     * End event handler triggered when client disconnects
     * @private
     * @memberof DaemonClient
     */
    end() {
        imperative_1.Imperative.api.appLogger.trace('daemon client disconnected');
    }
    /**
     * Close event handler triggered when client closes connection
     * @private
     * @memberof DaemonClient
     */
    close() {
        imperative_1.Imperative.api.appLogger.trace('client closed');
    }
    /**
     * Data event handler triggered for whenever data comes in on a connection
     * @private
     * @param {Buffer} data
     * @memberof DaemonClient
     */
    data(data) {
        const stringData = data.toString();
        // assess if this data is related to a prompt reply and ignore if so
        if (stringData.indexOf(DaemonClient.ZOWE_DEAMON_ID) > -1) {
            try {
                const jsonData = JSON.parse(stringData);
                if (jsonData.id && jsonData.id === DaemonClient.ZOWE_DEAMON_ID) {
                    return;
                }
            }
            catch (_a) {
                // do nothing
            }
        }
        // NOTE(Kelosky): this is not exposed yet, but will allow for a clean shut down if undocumented `--shutdown`
        // is written to the persistent Processor.  It should be wrapped in a new header and handled in DaemonClient.ts, e.g. x-zowe-daemon-shutdown
        const stopOffset = data.toString().indexOf(DaemonClient.STOP_KEY);
        if (stopOffset > -1) {
            if (this.mServer) {
                imperative_1.Imperative.api.appLogger.debug("shutting down");
                this.mClient.write(`Terminating server`);
                this.mClient.end();
                this.mServer.close();
            }
            // accept input parameters as we do without running in a server mode and pass our clients stream
            // handle as context
        }
        else {
            const [cwd, cmdInput] = stringData.split("\r", 2);
            imperative_1.Imperative.api.appLogger.trace(`daemon input command: ${cmdInput}`);
            imperative_1.Imperative.commandLine = cmdInput;
            // TODO Remove the --dcd hack - perhaps use PWD env var instead?
            // This will be a bigger problem when we need to support multitenancy
            imperative_1.Imperative.parse(`${cmdInput} --dcd ${cwd}`, { stream: this.mClient });
        }
    }
}
exports.DaemonClient = DaemonClient;
/**
 * Undocumented paramter to cause daemon to shutdown
 * @private
 * @static
 * @memberof DaemonClient
 */
DaemonClient.STOP_KEY = "--shutdown";
DaemonClient.ZOWE_DEAMON_ID = "daemon-client";
//# sourceMappingURL=DaemonClient.js.map